/*
 * HTTSQUASH.H
 * Contains ip range structure and the networking
 * fuctions used by the program.
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef HTTSQUASH_LIB
#define HTTSQUASH_LIB

#include <stdio.h>
#include <string.h>
#ifdef WIN32
#include <windows.h>
#include <winsock2.h>
#else
#include <netdb.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#endif

#ifdef WIN32
WORD version;
WSADATA WSAData;
#endif

#define MAX_DATA 2048
#define MAX_PROC 6

int count=0;

//ip range struct
struct iprange {
	char range[22];
	char netblock[20];
	char addr[22];
	int start;
	int end;
	int curr;
	int port;
	int time;
	int verbose;
	int cookiejar;
};


//portable open socket
int opensock(char *dest, short int port)
{

	struct sockaddr_in sin;		// socket stuff
	struct hostent *hostinfo;	// ...
	int sock;					// ...

	#ifdef WIN32
		version=MAKEWORD(1,1);
		WSAStartup(version, &WSAData);
	#endif

	//open socket
	if (!(sock=socket(AF_INET,SOCK_STREAM,0)))
		return -1;

	//resolve destination name
	if(!(hostinfo=gethostbyname(dest)))
		return -2;

	//set options
	sin.sin_family=AF_INET;
	sin.sin_addr=*(struct in_addr *)hostinfo->h_addr;
	sin.sin_port=htons(port);

	//connection...
	if (connect(sock,(struct sockaddr *)&sin, sizeof(sin)))
		return -3;

	return sock;
}

//portable close socket
void closesock(int sock)
{
	#ifdef WIN32
		closesocket(sock);
	#else
		close(sock);
	#endif
}

//portable send data
void senddata(int sock, const char *data)
{
	send(sock,data,strlen(data),0);
}

//portable recv data
char *recvdata(int sock, int max_data)
{
	int res=0;
	char *buffer=NULL;
	buffer = (char *) malloc(max_data);

	res = recv(sock,buffer,max_data,0);
	buffer[res] = '\0';

	return buffer;
}

//portable recv data loop
char *recvdataloop(int sock, int max_data)
{
	int res=0, val=0;
	char *pb=NULL, *buffer=NULL;
	pb= (char *) malloc(max_data+1);

	while(res = recv(sock,pb,max_data,0)) {
		buffer = (char *) realloc(buffer, val+res);
		buffer[val] = '\0';
		val+=res;
		
		pb[res] = '\0';
		strcat(buffer,pb);
		pb[0] = '\0';
	}

	free(pb);

	return buffer;
}

#endif /* HTTSQUASH_LIB */

