/*
 * ReverseRaider, domain scanner.
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "reverseraider.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>
#include <ctype.h>

#define MAX_PROC 6

/*
 * Catch SIGALRM
 */
void catch_alarm (int sig)
{
    exit(0);
}

/*
 * Brute force wordlist scanner
 */
void wordlistscanner(struct options *opt)
{
    char host[MAX_ARG];
    char prefix[MAX_ARG];
    int num=0, i=0, count=0;
    FILE *wordlist;

    struct hostent *resolve;

    wordlist = fopen(opt->wordlist, "r");
    if(!wordlist) {
        perror("Error: open wordlist");
        exit(0);
    }

    while(!feof(wordlist)) {
        fgets(prefix, MAX_ARG, wordlist);
        for(i=0; prefix[i]!=0; i++)
            if (prefix[i]=='\n') {
                prefix[i]='\0';
                break;
            }
    
        memset(host, MAX_ARG, 0);
    
        snprintf(host, MAX_ARG, "%s.%s", prefix, opt->domain);
    
        //child (scan..)
        if(fork()==0) {

            if((resolve = gethostbyname(host)) == NULL) {}
            else {
                printf("%s %s\n", host, inet_ntoa(*(struct in_addr *) resolve->h_addr));
                fflush(stdout);
            }

            if(opt->permutation) {
            
                for(i=0; i<10; i++) {
                    snprintf(host, MAX_ARG, "%s%d.%s", prefix, i, opt->domain);

                    if((resolve = gethostbyname(host)) == NULL) {
                        continue;
                    }
                    else {
                        printf("%s %s\n", host, inet_ntoa(*(struct in_addr *) resolve->h_addr));
                    }
    
                    fflush(stdout);
                }

                for(i=0; i<10; i++) {
                    snprintf(host, MAX_ARG, "%s0%d.%s", prefix, i, opt->domain);

                    if((resolve = gethostbyname(host)) == NULL) {
                        continue;
                    }
                    else {
                        printf("%s %s\n", host, inet_ntoa(*(struct in_addr *) resolve->h_addr));
                    }
    
                    fflush(stdout);
                }
    
                for(i=0; i<10; i++) {
                    snprintf(host, MAX_ARG, "%d%s.%s", i, prefix, opt->domain);

                    if((resolve = gethostbyname(host)) == NULL) {
                        continue;
                    }
                    else {
                        printf("%s %s\n", host, inet_ntoa(*(struct in_addr *) resolve->h_addr));
                    }
    
                    fflush(stdout);
                }

                for(i=0; i<10; i++) {
                    snprintf(host, MAX_ARG, "0%d%s.%s", i, prefix, opt->domain);

                    if((resolve = gethostbyname(host)) == NULL) {
                        continue;
                    }
                    else {
                        printf("%s %s\n", host, inet_ntoa(*(struct in_addr *) resolve->h_addr));
                    }
    
                    fflush(stdout);
                }
    
            }
    
            fflush(stdout);
    
            exit(0);
        }
    
        //daddy...
        else {
            if(count>=MAX_PROC) for(count; count>0; count--) wait(0);
        }

        count++;
    }
    
    for(count; count>0; count--) wait(0);
    
    fclose(wordlist);
}

/*
 * Reverse resolution scanner
 */
void reversescanner(struct options *opt)
{
    char addr[MIN_ARG];
    char **name;
    char *c;
    
    struct in_addr inaddr;
    struct hostent *resolve;

    int i=0, start, end, curr, count=0;
    
    start = opt->start;
    end = opt->end;
    curr = opt->start;
    
    while(curr <= end) {
    
        //child (scan..)
        if(fork()==0) {
        
            alarm(opt->time); //timer

            sprintf(addr, "%s.%u", opt->block, curr);
            inaddr.s_addr = inet_addr(addr);
            
            // Resolve hostname
            resolve = gethostbyaddr(&inaddr, sizeof (struct in_addr), AF_INET);
            
            if(resolve==NULL || resolve->h_name==NULL)
                exit(0);

            // Print result
            printf( "%s %s\n", addr, resolve->h_name );
            
            name=resolve->h_aliases;
            for(i=0; name[i]!=NULL; i++)
                printf("%s %s\n", addr, name[i]);
        
            exit(0);
        }
        
        //daddy...
        else {
            if(count>=MAX_PROC) for(count; count>0; count--) wait(0);
        }

        count++;
        curr++;
    }
    
    for(count; count>0; count--) wait(0);
}

/*
 * Program init
 */
int main(int argc, char **argv)
{
    struct options opt;

    // Establish a handler for SIGALRM signals
    signal (SIGALRM, catch_alarm);
    signal (SIGTERM, catch_alarm);
    signal (SIGKILL, catch_alarm);
    
    // parse options
    parseopt(&opt, argc, argv);
    
    // Wordlist scanning
    if(opt.domain[0]!='\0')
        wordlistscanner(&opt);
    
    // Reverse scanning
    else
        reversescanner(&opt);
    
    return 0;
}
